@@ -50,6 +50,7 @@ HD_DEFINE_PLUGIN_MODULE (GpsDataLoggerWidget, gps_data_logger_widget, HD_TYPE_HO
 static void update_status_and_buttons (GpsDataLoggerWidget * self);
 static gboolean manual_on_stop (gpointer data);
 static void on_stop(LocationGPSDControl *control, gpointer data);
+void gps_data_logger_update_distance(gpointer data);
 	
 /* -------------------------------------------------------------------- */
 
@@ -154,6 +155,11 @@ debuglog_start(GpsDataLoggerWidget * self)
 static void
 logfile_close(GpsDataLoggerWidget * self)
 {
+    if (self->filtered_file) {
+        g_fprintf(self->filtered_file, "</trkseg>\n</trk>\n</gpx>\n");
+        fclose(self->filtered_file);
+        self->filtered_file = NULL;
+    }
     if (self->file) {
         g_fprintf(self->file, "</trkseg>\n</trk>\n</gpx>\n");
         fclose(self->file);
@@ -172,6 +178,7 @@ logfile_start(GpsDataLoggerWidget * self)
     time_t current;
     struct tm *tp;
     char string_buf[5];
+    char *filtered_path;
 
     /* logfile_start can only be called in APP_STATE_STARTING state */
     if (self->app_state != APP_STATE_STARTING)
@@ -184,11 +191,52 @@ logfile_start(GpsDataLoggerWidget * self)
         self->has_comma_as_decimal_point = FALSE;
     }
     /* If the logfile is already open (should not happen) then close it */
-    if (self->file)
+    if ((self->file) || (self->filtered_file))
         logfile_close(self);
     /* Name the log file using the current time */
     current = time(NULL);
     tp = localtime(&current);
+
+    if (self->settings->save_filtered_log) {
+	    filtered_path = g_strdup_printf("%s/%d%02d%02d_%02d%02d%02d_filtered.gpx",
+			           self->settings->log_folder,
+			    	   tp->tm_year + 1900,
+		   		   tp->tm_mon + 1,
+			   	   tp->tm_mday,
+			   	   tp->tm_hour,
+			   	   tp->tm_min,
+			   	   tp->tm_sec );
+	    if (! filtered_path) {
+        	/* out of memory?? */
+	        hildon_banner_show_informationf(NULL,
+        	                NULL,
+                	        _("Could not open the filtered GPX file for writing!"));
+		self->filtered_file = NULL;
+	    } else {
+    		self->filtered_file = fopen(filtered_path, "w");
+	    	if (! (self->filtered_file)) {
+        		hildon_banner_show_informationf(NULL,
+                	        NULL,
+                        	_("Could not open \"%s\" for writing!"),
+				filtered_path);
+#ifndef __arm__
+			g_print(_("Could not open the filtered file for writing\n"));
+#endif
+		} else {
+        		g_fprintf(self->filtered_file, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
+	        	g_fprintf(self->filtered_file, "<gpx version=\"1.0\"\n");
+		        g_fprintf(self->filtered_file, "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
+		        g_fprintf(self->filtered_file, "creator=\"gps-data-logger on Maemo\"\n");
+		        g_fprintf(self->filtered_file, "xmlns=\"http://www.topografix.com/GPX/1/0\"\n");
+		        g_fprintf(self->filtered_file, "xsi:schemaLocation=\"http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd\">\n");
+		        g_fprintf(self->filtered_file, "<trk>\n");
+		        g_fprintf(self->filtered_file, "<trkseg>\n");
+		}
+	    }
+    } else {
+	self->filtered_file = NULL;
+    }
+	    
     if (self->path) {
 	g_free(self->path);
     }
@@ -270,6 +318,10 @@ update_status_and_buttons (GpsDataLoggerWidget * self)
 #ifndef __arm__
 		g_print(" starting: loading new icon, disabling rec\n");
 #endif
+	        /* if the screen is on, update the distance counter */
+	        if (self->display_status)
+	            gps_data_logger_update_distance(self);	
+
 		break;
 	case APP_STATE_RECORDING:
 		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
@@ -375,6 +427,8 @@ handle_button_press (GtkWidget *button,
 	     * segment */
 	    if (self->points_recorded_in_current_segment)
 	        g_fprintf(self->file, "</trkseg>\n<trkseg>\n");
+	    if (self->filtered_points_recorded_in_current_segment)
+	        g_fprintf(self->filtered_file, "</trkseg>\n<trkseg>\n");
 	    location_gps_device_reset_last_known(self->device);
             location_gpsd_control_start(self->control);
 	}
@@ -731,6 +785,7 @@ on_changed(LocationGPSDevice *device, gpointer data)
 #endif
                             update_status_and_buttons(self);
 	                    self->points_recorded_in_current_segment = 0;
+	                    self->filtered_points_recorded_in_current_segment = 0;
 			    self->auto_pause_time_mark = 0;
 			} else if ((self->app_state == APP_STATE_RECORDING) &&  /* self->app_state == APP_STATE_STARTING  or keep auto_pause? */
 				   (self->settings->auto_pause) &&
@@ -788,7 +843,21 @@ on_changed(LocationGPSDevice *device, gpointer data)
 
 			        /* if the screen is on, update the distance counter */
 			        if (self->display_status)
-			            gps_data_logger_update_distance(self);	
+			            gps_data_logger_update_distance(self);
+			        /* save the filtered log if needed */
+			        if (self->settings->save_filtered_log) {
+			            self->filtered_points_recorded_in_current_segment += 1;
+			            g_fprintf(self->filtered_file, string_buf);
+				    g_fprintf(self->filtered_file, 
+					      "  <time>%d-%02d-%02dT%02d:%02d:%02dZ</time>\n",
+					      tp->tm_year + 1900,
+					      tp->tm_mon + 1,
+					      tp->tm_mday,
+					      tp->tm_hour,
+					      tp->tm_min,
+					      tp->tm_sec);
+				    g_fprintf(self->filtered_file, "</trkpt>\n");
+				}	
 			    }
 #ifdef DEBUG
                             if (self->logfile) {
@@ -813,13 +882,14 @@ on_changed(LocationGPSDevice *device, gpointer data)
 			    }
 			    g_fprintf(self->file, string_buf);
 			}
-			g_fprintf(self->file, "  <time>%d-%02d-%02dT%02d:%02d:%02dZ</time>\n",
-					tp->tm_year + 1900,
-					tp->tm_mon + 1,
-					tp->tm_mday,
-					tp->tm_hour,
-					tp->tm_min,
-					tp->tm_sec);
+			g_fprintf(self->file, 
+				  "  <time>%d-%02d-%02dT%02d:%02d:%02dZ</time>\n",
+				  tp->tm_year + 1900,
+				  tp->tm_mon + 1,
+				  tp->tm_mday,
+				  tp->tm_hour,
+				  tp->tm_min,
+				  tp->tm_sec);
 			if (device->fix->mode == LOCATION_GPS_DEVICE_MODE_2D)
 				g_fprintf(self->file, "  <fix>2d</fix>\n");
 			else if (device->fix->mode == LOCATION_GPS_DEVICE_MODE_3D)
@@ -1117,6 +1187,7 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
 			_("osso_initialize failed"));
     }
     self->file = NULL;
+    self->filtered_file = NULL;
     self->path = NULL;
 #ifdef DEBUG
     self->logfile = NULL;
