@@ -69,11 +69,13 @@ struct _GpsDataLoggerWidget
 	gboolean	has_comma_as_decimal_point;
 	gint	 	app_state;
 	gint		points_recorded_in_current_segment;
+	gint		filtered_points_recorded_in_current_segment;
 	double		auto_pause_time_mark;
 	LocationGPSDControl *control;
 	LocationGPSDevice *device;
 	DBusConnection	*dbus_conn;
 	FILE		*file;
+	FILE		*filtered_file;
 	gchar		*path;
 #ifdef DEBUG
 	FILE		*logfile;
